/*

 NYTimes GraphMaker
 http://blog.blprnt.com/blog/blprnt/7-days-of-source-day-6-nytimes-graphmaker
 
 February, 2009
 blprnt@blprnt.com
 
 This is a really simple sketch to make standard bar graphs & radial bar graphs showing word usage trends in the NYTimes.
 This sketch is very easy to use - just enter your own words in where indicated below.
 
 NOTE: Make sure you enter your NYTimes Article Search API on the TimesArticleSearch tab.
 
 Key controls:
 't' - toggle between radial and standard graph
 's' - save out a .PNG
 'p' - save out a .PDF
 
 */

//FIRST, LET'S IMPORT SOME CLASSES
import toxi.math.noise.*;
import toxi.math.waves.*;
import toxi.geom.*;
import toxi.math.*;
import toxi.math.conversion.*;
/*import toxi.geom.util.*;*/ // had to be removed.

import processing.opengl.*;
import org.json.*;
import java.util.ArrayList;
import java.util.AbstractCollection;

import processing.pdf.*;

//NOW, LET'S DECLARE SOME VARIABLES
                                                                            //vv THIS IS ALL OF THE EASY STUFF TO CHANGE AND EXPLORE
int s = 1985;                                                               // - Start year (this can go as far back as 1981)
int e = 2009;                                                               // - End year
int maxcount = 0;

String[] words = {"cocaine", "heroin","crack+cocaine","meth"};             // - <<< ADD YOUR OWN WORDS HERE
color[] colors = {#FF9900,#FF0000, #1494CC,  #FFE11E, #FF216E};             // - <<< ADD YOUR OWN COLOURS HERE
color textColor = #15302E;                                                  // - Text color
color backColor = #EFEFEF;                                                  // - Background color
boolean isRadial = true;                                                   // - Is the graph radial (true)? Or a normal bar graph?(false)

float graphHeight = 500;
float graphWidth = 1000;
Vec2D start = new Vec2D(0,0);
Vec2D tstart;

//This is the TimesArticleSearch object that does most of the work
TimesArticleSearch search;
//This HashMap is used to store search results
HashMap results = new HashMap();

//Font object for the text key
PFont font;

//The graphs can be toggled between radial graphs and standard bar graphs. This transition is guided by these variables
float circleness = 0;  // What is the circleness right now?
float tcircleness = 1; // What is the target circleness?

//This boolean (true/false) value is used to turn on PDF recording for single frames
boolean PDFing = false;

//LET'S SET SOME STUFF UP
void setup() {
  
  //Set the size and colour of the sketch
  size(1000,800);
  background(backColor);
  smooth();
  
  //Load the font to be used in text display
  font = createFont("Helvetica", 24); 
  textFont(font); 
  
  //Starting point changes depending on wether the graph is radial or not
  setCenter();
  
  //The guts of the work is done by a TimesArticleSearch object
  search = new TimesArticleSearch();
  //Which we tell not to return any fields (like headlines, etc.) for brevity
  search.addFields("+");
  //We also ask for the publication month facet, which allows us to use 1 server call for a year, instead of one for every month. Clever!
  search.addFacets("publication_month"); 
  
  //Populate data banks using the TimesArticleSearch object (details are in the getResultsForTerm method below)
  int i;
  String w;
  for (i=0; i < words.length; i++) {
     w = words[i];
     search.clearQueries();
     search.addQueries(w);
     results.put(w, getResultsForTerm(w));
  };  
  
};

//THIS IS THE STUFF WE DO ONCE PER FRAME
void draw() {
  //Paint the screen white
  if (PDFing) beginRecord(PDF, words[0] + "_" + hour() + "_" + minute() + "_" + second() + ".pdf");
  background(255);
  //Adjust the circleness & the starting point
  circleness += (tcircleness - circleness)/8;
  start.interpolateToSelf(tstart, 0.2);
  //Draw the graph
  int i;
  String w;
  for (i=0; i < words.length; i++) {
      w = words[i];
      int[][] data = (int[][]) results.get(w);
      drawGraph(data, colors[i]);
  };
  //Draw the text key
  drawKey();
  if (PDFing) {
    PDFing = false;
    endRecord();
  };
};

/* ---------------------------
setCenter
- Put the center point in the center if it's a radial graph, and near the bottom left if it's a standard graph
-----------------------------*/

void setCenter() {
  tstart = (isRadial) ? (new Vec2D(width/2,height/2)):(new Vec2D(0,700));
};

/* ---------------------------
getResultsForTerm
- This is the function that does the searches, and stores the results in a set of matrixes, which are then filed in the HashMap
-----------------------------*/


int[][] getResultsForTerm(String term) {
  //Clear the search object from the previous search
  search.clearQueries();
  //Add a single query - the 
  search.addQueries(term);
  //Create an empty matrix with slots for each month in each year
  int[][] years = new int[(e - s) + 1][12];
  int c = 0;
  for (int i = s; i <= e; i++) {
    //Let us know how the search is going, won't you?
    println(term + " - " + i + ":" + e);
    //Convert the year count to a string and pass it to the search object
    String ys = str(i);
    search.clearFacetQueries();
    //We use the publication_year facet to restrict searches to a specific year
    search.addFacetQuery("publication_year", ys);
    //A TimesArticleSearchResult object is returned from the search
    TimesArticleSearchResult r = search.doSearch();
    //We retrieve an array of TimesFacetObjects from the result - these individual TFOs hold the numbers that we need
    TimesFacetObject[] months = r.getFacetList("publication_month");
    for (int j=0; j < months.length; j++) {
      TimesFacetObject m = months[j];
      //We populate the year matrix with the numbers pulled out of the TFO
      years[c][int(m.term) - 1] = m.count;
      //We keep track of the global max, in order to scale everything correctly
      maxcount = max(m.count, maxcount);
    };
    c++;
  };
  //Finally, we send the matrix we've created back
  return(years);
};

/* ---------------------------
drawGraph
- Renders the data points on screen as a set of bars
- Each term is rendered as a separate graph
-----------------------------*/

void drawGraph(int[][] data, color c) {
  //There's some variation in colour - we want this to be the same every frame so we set a random seed
  randomSeed(2);
  //We're going to translate to the center point of the graph
  pushMatrix();
  translate(start.x, start.y);
  noStroke();                                                                  //comment this out if you want outlines on the bars
  //How many years are we rendering in total?
  int yrs = (e -s) + 1;
  //How far apart will the bars be spaced in a standard graph?
  float step = graphWidth / (yrs * 12);
  //How many degrees apart will the bars be in a radial graph?
  float rinc = ((2 * PI) / (yrs * 12));
  //Loop through the years
  for (int i = 0; i < yrs; i++) {
    //Loop through the months
    for (int j = 0; j < 12; j++) {
      //In radial graphs (circleness = 1), we rotate
      rotate(rinc * circleness);
      //In standard graphs (circleness = 0), we separate
      translate(step * (1 - circleness), 0);
      
      //Set the colour of the bar segments
      color newc = color(red(c), green(c), blue(c), random(200,240));
      fill(newc);

      //How many units does the bar represent?
      int localcount = data[i][j];
      //What fraction of the max value is this?
      float h = localcount / float(maxcount);
      
      //Draw the bars!
      rect(0, 0, step, -h * graphHeight);

    };
  };
  popMatrix();
};

/* ---------------------------
drawKey
- Renders the text key
-----------------------------*/

void drawKey() {
  textFont(font);
  textSize(18);
  for (int i = 0; i < words.length; i++) {
    fill(colors[i]);
    text(words[i],25, 32 + (i * 20));
  };
  fill(textColor);
  text(s + "-" + e, 25, 32 + (words.length * 20));  
};

/* ---------------------------
keyPressed handler
- Listens for key presses and does key-pressy things.
-----------------------------*/
void keyPressed() {
  if (key == 't') {
    isRadial = !isRadial;
    tcircleness = (isRadial) ? (1):(0);
    setCenter();
  };
  if (key == 's') {
    save(words[0] + "_" + hour() + "_" + minute() + "_" + second() + ".png");
  };
  if (key == 'p') {
    PDFing = true;
  };
};


